﻿using System;
using System.ComponentModel;
using System.Windows.Input;

namespace WpfApplication5
{
    public class BaseViewModel : NotifyPropertyChanged, IDisposable
    {
        #region Events
        public event EventHandler<EventArgs> AddCommandExcuted;
        public event EventHandler<EventArgs> DeleteCommandExcuted;
        public event EventHandler<EventArgs> UpdateCommandExcuted;
        public event EventHandler<EventArgs> ApprovalCommandExcuted;
        public event EventHandler<EventArgs> AcceptCommandExcuted;
        public event EventHandler<EventArgs> CloseCommandExcuted;
        #endregion  // Events

        #region Member Fields
        private bool _disposed = false;
        #endregion  // Member Fields

        #region Constructor
        public BaseViewModel() { }
        #endregion  // Constructor

        #region Destructor
        ~BaseViewModel()
        {
            this.DoDispose(false);
        }
        #endregion  // Destructor

        #region Propeties
        protected bool IsDeleteCheck
        {
            get;
            private set;
        }
        #endregion  // Propeties

        #region Commands
        private DelegateCommand<object> _addCommand;
        /// <summary>
        /// Gets the add command.
        /// </summary>
        public ICommand AddCommand
        {
            get
            {
                return _addCommand ??
                       (_addCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteAddCommand(param),
                            param => this.CanExecuteAddCommand()));
            }
        }

        private DelegateCommand<object> _deleteCommand;
        /// <summary>
        /// Gets the delete command.
        /// </summary>
        public ICommand DeleteCommand
        {
            get
            {
                return _deleteCommand ??
                       (_deleteCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteDeleteCommand(param),
                            param => this.CanExecuteDeleteCommand()));
            }
        }

        private DelegateCommand<object> _updateCommand;
        /// <summary>
        /// Gets the update command.
        /// </summary>
        public ICommand UpdateCommand
        {
            get
            {
                return _updateCommand ??
                       (_updateCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteUpdateCommand(param),
                            param => this.CanExecuteUpdateCommand()));
            }
        }

        private DelegateCommand<object> _approvalCommand;
        /// <summary>
        /// Gets the approval command.
        /// </summary>
        public ICommand ApprovalCommand
        {
            get
            {
                return _approvalCommand ??
                       (_approvalCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteApprovalCommand(param),
                            param => this.CanExecuteApprovalCommand()));
            }
        }

        private DelegateCommand<object> _acceptCommand;
        /// <summary>
        /// Gets the accept command.
        /// </summary>
        public ICommand AcceptCommand
        {
            get
            {
                return _acceptCommand ??
                       (_acceptCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteAcceptCommand(param),
                            param => this.CanExecuteAcceptCommand()));
            }
        }

        private DelegateCommand<object> _closeCommand;
        /// <summary>
        /// Gets the close command.
        /// </summary>
        public ICommand CloseCommand
        {
            get
            {
                return _closeCommand ??
                       (_closeCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteCloseCommand(param),
                            param => this.CanExecuteCloseCommand()));
            }
        }

        private DelegateCommand<object> _searchCommand;
        /// <summary>
        /// Gets the search command.
        /// </summary>
        public ICommand SearchCommand
        {
            get
            {
                return _searchCommand ??
                       (_searchCommand =
                        new DelegateCommand<object>(
                            param => this.ExecuteSearchCommand(param),
                            param => this.CanExecuteSearchCommand()));
            }
        }
        #endregion  // Commands

        #region Command Methods
        protected virtual void ExecuteAddCommand(object param)
        {
            if (AddCommandExcuted != null)
                AddCommandExcuted(this, new EventArgs());
        }

        protected virtual bool CanExecuteAddCommand()
        {
            return true;
        }

        protected virtual void ExecuteDeleteCommand(object param)
        {
            if (IsDeleteCheck != true)
            {
                return;
            }

            if (DeleteCommandExcuted != null)
                DeleteCommandExcuted(this, new EventArgs());
        }

        protected virtual bool CanExecuteDeleteCommand()
        {
            return true;
        }

        protected virtual void ExecuteUpdateCommand(object param)
        {
            if (UpdateCommandExcuted != null)
                UpdateCommandExcuted(this, new EventArgs());
        }

        protected virtual bool CanExecuteUpdateCommand()
        {
            return true;
        }

        protected virtual void ExecuteApprovalCommand(object param)
        {
            if (ApprovalCommandExcuted != null)
                ApprovalCommandExcuted(this, new EventArgs());
        }

        protected virtual bool CanExecuteAcceptCommand()
        {
            return true;
        }

        protected virtual void ExecuteAcceptCommand(object param)
        {
            if (AcceptCommandExcuted != null)
                AcceptCommandExcuted(this, new EventArgs());
        }

        protected virtual bool CanExecuteApprovalCommand()
        {
            return true;
        }

        protected virtual void ExecuteCloseCommand(object param)
        {
            if (CloseCommandExcuted != null)
                CloseCommandExcuted(this, new EventArgs());
        }

        protected virtual bool CanExecuteCloseCommand()
        {
            return true;
        }

        protected virtual void ExecuteSearchCommand(object param)
        {
            throw new NotImplementedException();
        }

        protected virtual bool CanExecuteSearchCommand()
        {
            return true;
        }
        #endregion  // Command Methods

        #region Disposable
        public virtual void Dispose()
        {
            Console.WriteLine("BaseViewModel Dispose");
            this.DoDispose(true);
            GC.SuppressFinalize(this);
        }

        public virtual void DoDispose(bool disposing)
        {
            try
            {
                if (!_disposed)
                {
                    if (disposing)
                    {
                        this.Nothing();
                    }

                    _disposed = true;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("BaseViewModel Dispose Exception : " + ex);
                throw;
            }
            finally
            {
                Console.WriteLine("BaseViewModel Dispose Completion");
            }
        }

        public void Nothing()
        {
            this.ClearAllPropertyChangedHandlers();
        }
        #endregion  // Disposable
    }
}
