﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Input;

namespace WpfApplication5
{
    public class DelegateCommand<T> : ICommand
    {
        #region Member Fields
        private readonly Predicate<T> _canExecute;
        private readonly Action<T> _execute;
        #endregion

        #region Construct
        public DelegateCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        public DelegateCommand(Action<T> execute, Predicate<T> canExecute)
        {
            if (execute == null)
            {
                throw new ArgumentNullException("executeMethod");
            }

            _execute = execute;
            _canExecute = canExecute;
        }
        #endregion

        #region ICommand 구현
        public event EventHandler CanExecuteChanged
        {
            add
            {
                CommandManager.RequerySuggested += value;
            }
            remove
            {
                CommandManager.RequerySuggested -= value;
            }
        }

        public bool CanExecute(object parameter)
        {
            if (_canExecute == null)
                return true;

            return _canExecute((parameter == null) ?
                default(T) : (T)Convert.ChangeType(parameter, typeof(T)));
        }

        public void Execute(object parameter)
        {
            _execute((parameter == null) ? default(T) : (T)Convert.ChangeType(parameter, typeof(T)));
        }
        #endregion
    }
}
